package aracloud;

import java.io.File;
import java.nio.file.Path;
import static java.nio.file.StandardWatchEventKinds.*;
import java.nio.file.*;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *  Esta clase permite monitorear las acciones sobre los archivos en
 *  una determinada carpeta. Las 3 acciones básicas son: se crea, modifica
 *  o elimina un archivo/carpeta. Esta clase permite la sincronización en
 *  tiempo real de los archivos de una carpeta compartida con el servidor
 *  Aragorn. Es necesario entonces que esta clase sea ejecutada en un hilo
 *  para poder mantener el paralelismo entre ejecuciones en el programa.
 *
 * @author marcelo
 */
public class FileMonitor extends Thread{

    /* El objeto newWatcher permite asociar directorios y
    *  archivos que se desean monitorear. Es de interés que estos
    *  sean personalizados a gusto del cliente
    */
    private final WatchService newWatcher;

    /* El objeto scp permite crear una instancia que facilite tanto la
    * transferencia como la comunicación entre el host local y el servidor
    * remoto
    */

    private final SCPFileSender scp;
    private String folder;
    private String remoteFolder;

    public FileMonitor(WatchService watcher, SCPFileSender f, String folder, String remoteFolder){
        newWatcher = watcher;
        scp = f;
        this.folder = folder;
        this.remoteFolder = remoteFolder;
    }

    @Override
    public void run(){

        while (true) {
            WatchKey key;
            try {
                key = newWatcher.take();
            }
            catch (InterruptedException ex) {
                return;
            }

            for (WatchEvent<?> event : key.pollEvents()) {
                // get event type
                WatchEvent.Kind<?> kind = event.kind();

                // get file name
                @SuppressWarnings("unchecked")
                WatchEvent<Path> ev = (WatchEvent<Path>) event;

                Path fileName = ev.context();

                if (kind == OVERFLOW) {
                    continue;
                } else if (kind == ENTRY_MODIFY || kind == ENTRY_CREATE) {
                    SCPFileSender.Update newFile = null;
                    try{
                        newFile = scp.new Update(folder, fileName.toString(), remoteFolder);
                        newFile.start();
                        newFile.join();

                        if((new File(folder + "/" + fileName)).isDirectory()){
                            WatchService watcher = FileSystems.getDefault().newWatchService();
                            Path dir = Paths.get(folder + "/" + fileName);
                            dir.register(watcher, ENTRY_CREATE, ENTRY_DELETE, ENTRY_MODIFY);
                            FileMonitor monitor = new FileMonitor(watcher, scp, folder + fileName + "/", remoteFolder + "/" + fileName.toString());
                            monitor.start();
                        }

                    }
                    catch(Exception e){}

                    //cm.updateDataFile();
                    } else if (kind == ENTRY_DELETE) {
                        SCPFileSender.Delete deleteFile = scp.new Delete(fileName.getFileName().toString(),remoteFolder);
                        deleteFile.start();
                        try{
                            deleteFile.join();
                        }
                        catch(Exception c){

                        }
                    //cm.updateDataFile()
                }
                    /*else if(kind == ENTRY_MODIFY){
                        SCPFileSender.Update newFile = null;
                        try{

                            newFile = scp.new Update(folder, fileName.toString(), remoteFolder);
                            newFile.start();
                            newFile.join();

                        }
                        catch(Exception e){System.out.println(e);}
                        }*/
            }

            // IMPORTANT: The key must be reset after processed
            boolean valid = key.reset();
            if (!valid) {
                break;
            }
        }
    }

}
